<?php
require_once('config.php');
/**
 * サイトマップジェネレーター
 *
 * 簡単にサイトマップが生成できます。
 *
 *
 * Licensed under The MIT License
 * Redistributions of files must retain the above copyright notice.
 *
 * @author     Tadashi Suyama<suyama@its-office.jp>
 * @copyright  Copyright 2012, ITS, Inc. (http://www.its-office.jp/)
 * @package    sitemapgenerator
 * @since      PHP 5.3 
 * @version    v 1.0 -2012/03/07 10:15:23-
 * @license    MIT License (http://www.opensource.org/licenses/mit-license.php)
 */

class sitemap {

	protected $_host;
	protected $_path;
	protected $_ext;
	protected $_remove_dirs;
	protected $_remove_files;
	protected $_modify_names;
	protected $_title_opt;
	protected $_debug = false;
	
	public function __construct($debug = false)
	{
		$this->config();
		$this->setDebug($debug);
	}
	/**
 	* func config.
	*
	* @access protected
	* @since 1.0
	* @return 設定読み込み
	*/
	protected function config()
	{
		$config = new config();
		$this->_host         = $config->_host;
		$this->_path         = $config->_path;
		$this->_ext          = $config->_ext;
		$this->_remove_dirs  = $config->_remove_dirs;
		$this->_remove_files = $config->_remove_files;
		$this->_modify_names = $config->_modify_names;
		$this->_title_opt    = $config->_title_opt;

	}
	
	/**
 	* func setDebug.
	*
	* @access protected
	* @since 1.0
	* @return デバッグモードを設定
	*/
    protected function setDebug($debug){
	
        if ($debug) {
            $this->debug = true;
            ini_set('display_errors', 1);
            error_reporting(-1);
        } else {
            $this->debug = false;
            ini_set('display_errors', 0);
        }
    }
	
	/**
 	* func getDirFile.
	*
	* @access protected
	* @since 1.0
	* @return DirectoryIteratorクラスによるディレクトリ参照方法
	*/
	protected function getDirFileList($dir, $check = 'default')
	{
		
		$files = new DirectoryIterator($this->escape($dir));
		
		switch($check) {
			case 'file':
			foreach ($files as $file) {
				if ($file->isFile()) {
					$dirs[] = mb_convert_encoding($file->getFileName(), mb_internal_encoding(), 'ASCII,JIS,UTF-8,EUC-JP,SJIS');
				}
			}
			break;
			
			case 'dir':
			foreach ($files as $file) {
				if (!$file->isDot()) {
					if ($file->isDir()) {
						$dirs[] = mb_convert_encoding('../' . $file->getFileName() . '/', mb_internal_encoding(), 'ASCII,JIS,UTF-8,EUC-JP,SJIS');
					}
				}
			}
			if (!empty($dirs)) {
				array_unshift($dirs, '../');	
			}
			break;
			
			default:
			foreach ($files as $file) {
				if (!$file->isDot()) {
					$dirs[] = mb_convert_encoding($file->getFileName(), mb_internal_encoding(), 'ASCII,JIS,UTF-8,EUC-JP,SJIS');	
				}
			}
		}
		
		if (empty($dirs)) {
			$dirs = array();
		} else {	
		  	// 除外ファイル・フォルダオプション設定
			$dirs = array_diff($dirs, $this->_remove_dirs);
			$dirs = array_diff($dirs, $this->_remove_files);
		}

		return $dirs;
	}
	
	/**
 	* func getDirList.
	*
	* @access protected
	* @since 1.0
	* @return ディレクトリリスト取得
	*/
	protected function getDirList($dir)
	{
		return $this->getDirFileList($dir, 'dir');
	}
	
	/**
 	* func getFileList.
	*
	* @access protected
	* @since 1.0
	* @return ファイルリスト取得
	*/
	protected function getFileList($dir)
	{
		return $this->getDirFileList($dir, 'file');
	}
	
	
	/**
 	* func getFileList.
	*
	* @access protected
	* @since 1.0
	* @return ディレクトリーからのファイル一覧取得
	*/
	protected function getList($dir)
	{	
		$links = array();
				
		$files = $this->getFileList($dir);
		if (!empty($files)) {
			foreach ($files as $file) {
				$links[] = $this->getContents($dir, $file);
			} 
		} 
		
		return $links;
	}
	
	/**
 	* func getContents.
	*
	* @access protected
	* @since 1.0
	* @return ファイルの中身のタイトルタグを取得
	*/
	protected function getContents($dir, $file)
	{
		mb_language('Japanese');
		$parts = pathinfo($dir . $file);
		if (in_array($parts['extension'], $this->_ext)) {
			$size = filesize($dir . $file);
			if ($size < 104857600) {
				$directory = $this->removedDot($dir);
				$contents = file_get_contents($dir . $file, FILE_USE_INCLUDE_PATH); 
				$contents = mb_convert_encoding($contents, mb_internal_encoding(), 'ASCII,JIS,UTF-8,EUC-JP,SJIS'); 
				// タイトルタグの値のみ取得
				if (preg_match('#<title>(.*?)</title>#i', $contents, $hit)) { 
					$title = $hit[1];
					$link = '<a href="' . $this->removedSlash($this->_host) . $directory . $file . '">' . $this->getTitle($title) . '</a>';
				} else {
					$link = "";
				}
			} else {
				$link = "";
			}
		} else {
			$link = "";
		}
		 	return $link;
	}
	
	/**
 	* func removedDot.
	*
	* @access protected
	* @since 1.0
	* @return ドットを取り除く
	*/	
	protected function removedDot($dir)
	{
		$pattern = '#\.+\/#'; 
		$dir = preg_replace($pattern, '/', $dir);
		return $dir ;
	}
	
	/**
 	* func removedSlash.
	*
	* @access protected
	* @since 1.0
	* @return 最後のスラッシュを取り除く
	*/	
	protected function removedSlash($host)
	{
		return rtrim($host, '/');
	}
	
	/**
 	* func getTitle.
	*
	* @access protected
	* @since 1.0
	* @return ｜区切りのタイトルの前or後取得
	*/	
	protected function getTitle($str)
	{
		$str   = mb_ereg_replace('｜', '|', $str);
		$title = explode('|', $str);
		$num   = count($title);
		switch ($this->_title_opt) {
			case 'pre':
				$get_title = $title[0];
			break;
			case 'last':
				$get_title = $title[$num - 1];
			break;
			default:
				$get_title = $title[1];
			break;
		}
		return $get_title;
	}
	
	/**
 	* func display.
	*
	* @access public
	* @since 1.0
	* @return サイトマップとして表示
	*/
	public function display()
	{
		$dirs = $this->getDirList($this->escape($this->_path));
	
		if (is_array($dirs)) {
			foreach ($dirs as $dir) {
				$files = $this->getList($dir);
				$files = array_filter($files, 'strlen');
				foreach ( $this->_modify_names as $key => $name) {
					if ($dir === $key) {
						$title = $name;
						break;
					} else {
						$title = $dir;
					}
				}		
				echo '<ul class="smg">' . "\n" . '<li>' . $title ;
				if (!empty($files)) { 
				 	echo '<ul class="smg_nests">' . "\n";
					foreach ($files as $file) {
						echo '<li class="dir">' . $file . '</li>' . "\n"; 
						$out = true; 
					}
				} else {
					echo '</li>' . "\n" . '</ul>' . "\n";
					$out = false;
				}
				if ($out === true) {
					echo '</ul>' . "\n" . '</li>'. "\n" . '</ul>' . "\n";
				}
			}
		}
	}

	/**
 	* func escape.
	*
	* @access public
	* @since 1.0
	* @return 特殊文字をHTMLエンティティに変換する
	*/
	public function escape($string)
	{
		return htmlspecialchars($string, ENT_QUOTES, 'UTF-8');
	}
	
	/**
 	* func formatBytes.
	*
	* @access protected
	* @since 1.0
	* @return 容量を単位で表す
	*/
	protected function formatBytes($bytes)
	{
    	if ($bytes < 1024) {
        	return $bytes .' B';
    	} elseif ($bytes < 1048576) {
        	return round($bytes / 1024, 2) .' KB';
    	} elseif ($bytes < 1073741824) {
        	return round($bytes / 1048576, 2) . ' MB';
    	} elseif ($bytes < 1099511627776) {
        	return round($bytes / 1073741824, 2) . ' GB';
    	} else {
        	return round($bytes / 1099511627776, 2) .' TB';
    	}
	}

}

$sitemap = new sitemap(false);